import re
import urllib
import urllib2
import logging
import json

from django.utils.translation import ugettext_lazy as _
from django.db import models
from passerelle.base.models import BaseResource
from passerelle.sms import SMSGatewayMixin
from django.core.urlresolvers import reverse

from .choosit import ChoositRegisterWS


class ChoositError(Exception):
    pass

class ChoositSMSGateway(BaseResource, SMSGatewayMixin):
    key = models.CharField(max_length=64)
    default_country_code = models.CharField(max_length=3, default=u'33')
    # FIXME: add regexp field, to check destination and from format

    class Meta:
        verbose_name = 'Choosit'
        db_table = 'sms_choosit'

    @classmethod
    def get_verbose_name(cls):
        return cls._meta.verbose_name

    @classmethod
    def get_icon_class(cls):
        return 'phone'

    def send_msg(self, text, sender, destinations):
        """Send a SMS using the Choosit provider"""
        # from http://sms.choosit.com/documentation_technique.html
        # unfortunately it lacks a batch API...
        destinations = self.clean_numbers(destinations,
                self.default_country_code, prefix='')
        for dest in destinations:
            params = {
                'key': self.key,
                'recipient': dest,
                'content': text[:160],
                }
            try:
                data = urllib.urlencode([('data', json.dumps(params))])
                r = urllib2.Request('http://sms.choosit.com/webservice', data)
                p = urllib2.urlopen(r, timeout=5)
                output = json.load(p)
                p.close()
            except Exception, e:
                logging.exception(u'%s unable to send message %s to %s',
                        self, text, destinations)
                raise ChoositError(e)
            if 'error' in output:
                raise ChoositError(output['error'].encode('utf-8'))
            return { 'message': output.get('result') }


class ChoositRegisterNewsletter(models.Model):
    name = models.CharField(max_length=16)
    description = models.CharField(max_length=128, blank=True)

    class Meta:
        verbose_name = 'Choosit Registration'
        db_table = 'newsletter_choosit'

    def __unicode__(self):
        return u'%s: %s' % (self.name, self.description or u'<unnamed>')


class ChoositRegisterGateway(BaseResource):
    category = _('Newsletter registration')

    url = models.CharField(max_length=200)
    key = models.CharField(max_length=64)
    newsletters = models.ManyToManyField(ChoositRegisterNewsletter,
        blank=True)

    class Meta:
        verbose_name = 'Choosit Registration'
        db_table = 'registration_choosit'

    @classmethod
    def get_verbose_name(cls):
        return cls._meta.verbose_name

    @classmethod
    def get_icon_class(cls):
        return 'ressources'

    def get_absolute_url(self):
        return reverse('choosit-register-view', kwargs={'slug': self.slug})

    @classmethod
    def get_add_url(cls):
        return reverse('choosit-register-add')

    def get_edit_url(self):
        return reverse('choosit-register-edit', kwargs={'slug': self.slug})

    def get_delete_url(self):
        return reverse('choosit-register-delete', kwargs={'slug': self.slug})

    def get_list(self, user):
        reg = ChoositRegisterWS(self.url, self.key)

        ws = reg.subscriptions(email=user or '')
        if 'error' in ws:
            raise ChoositError(ws['error'])
        newsletters = ws['newsletter']
        subscriptions = ws['subscriptions']

        filters = dict([(n.name, n.description) for n in self.newsletters.all()])
        if filters:
            for name in newsletters.copy():
                if name not in filters:
                    del newsletters[name]
                elif filters[name]:
                    newsletters[name] = filters[name]

        ret = []
        for id, name in newsletters.items():
            newsletter = {
                        'name': id,
                        'description': name,
                        'transports': { 'available': ['mail'] }
                    }
            if user:
                if id in subscriptions:
                    newsletter['transports']['defined'] = ['mail']
                else:
                    newsletter['transports']['defined'] = []
            ret.append(newsletter)

        return ret

    def post(self, user, data):
        subscriptions = [row['name'] for row in data if row['transports']]
        reg = ChoositRegisterWS(self.url, self.key)
        ws = reg.update(subscriptions, user)
        return {"message": ws['status']}
