from django.views.generic.detail import SingleObjectMixin
from django.views.generic.base import View

from passerelle import utils

from .models import CsvDataSource


class CsvDataView(View, SingleObjectMixin):
    model = CsvDataSource

    def _filters_builder(self, request):
        filters = {}
        obj = self.get_object()

        params = request.GET

        case_insensitive = 'case-insensitive' in params
        query = params.get('q', None)

        if query:
            if case_insensitive:
                filters['text__icontains'] = query.lower()
            else:
                filters['text__contains'] = query

        # builds filters according to csv file header
        for column_title in [t.strip() for t in obj.columns_keynames.split(',') if t]:
            match = filter(
                (lambda ct: lambda x: x.startswith(ct))(column_title), params.keys()
            )
            for key in match:
                if case_insensitive:
                    filters[key] = params[key].lower()
                else:
                    filters[key] = params[key]

        if 'text' in filters:
            if case_insensitive:
                filters['text__ieq'] = filters['text'].lower()
            else:
                filters['text__eq'] = filters['text']
            filters.pop('text')

        return filters


    @utils.protected_api('can_access')
    @utils.to_json('api')
    def get(self, request, *args, **kwargs):
        obj = self.get_object()
        filters = self._filters_builder(request)
        return obj.get_data(filters)
