# Passerelle - uniform access to data and services
# Copyright (C) 2015  Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import json
import urlparse
import requests
from subprocess import Popen
import base64
import tempfile
import logging
import shutil
import magic

from django.views.generic import DetailView as GenericDetailView, View
from django.views.decorators.csrf import csrf_exempt

from passerelle import utils

# from .soap import get_client, client_to_jsondict, recursive_asdict
from .models import IParapheur

log = logging.getLogger(__name__)


class IParapheurDetailView(GenericDetailView):
    model = IParapheur
    template_name = 'passerelle/contrib/iparapheur/detail.html'


class DetailView(GenericDetailView):
    model = IParapheur

    def get_data(self, request, *args, **kwargs):
        raise NotImplementedError

    @utils.protected_api('can_access')
    @utils.to_json('api')
    def get(self, request, *args, **kwargs):
        return self.get_data(request, *args, **kwargs)



class PingView(DetailView):
    def get_data(self, request, *args, **kwargs):
        return self.get_object().ping()


class TypesView(DetailView):

    def get_data(self, request, *args, **kwargs):
        return [{'id': t, 'text': t} for t in self.get_object().get_types()]


class SubTypesView(DetailView):

    def get_data(self, request, *args, **kwargs):
        typ = request.GET.get('type')
        return [{'id': t, 'text': t} for t in self.get_object().get_subtypes(typ)]


class ListFiles(DetailView):
    def get_data(self, request, *args, **kwargs):
        status = request.GET.get('status')
        return [{'status': r.status, 'id': r.nom, 'timestamp': r.timestamp} \
                for r in self.get_object().get_files_list(status)]


class CreateFile(GenericDetailView):
    model = IParapheur

    @csrf_exempt
    def dispatch(self, request, *args, **kwargs):
        return super(CreateFile, self).dispatch(request, *args, **kwargs)

    @utils.protected_api('can_access')
    @utils.to_json('api')
    def post(self, request, *args, **kwargs):
        data = json.loads(request.body)
        title = data['title']
        typ = data['type']
        subtype = data['subtype']
        email = data.get('email')
        visibility = data['visibility']
        content = data['data']

        mime = magic.open(magic.MAGIC_MIME_TYPE)
        mime.load()
        content_type = data.get('content_type') if data.get('content_type') \
                else mime.buffer(base64.b64decode(data['data']))
        return self.get_object().create_file(typ, subtype, title,
                                visibility, content, content_type, email)


class FileView(DetailView):
    def get_data(self, request, *args, **kwargs):
        return self.get_object().get_file(kwargs['file_id'])


class FileStatusView(DetailView):
    def get_data(self, request, *args, **kwargs):
        return self.get_object().get_file_status(kwargs['file_id'])
